from pylab import *
from bisect import *
import dolfin

def integrate(f, I, k, method):
    """Returns the integral value of f over I.

    f: function to integrate, I: float array for time interval, k:
    float for time-step value and method: string for integration
    method ("Euler" or "Midpoint"). Returns float.
    """
    integral = 0.0
    for t in arange(I[0], I[1], k):
        t0 = t - k
        # Fill in implementation
        integral += 0.0

    return integral

def integrate2D(f, mesh):
    """Returns the integral value of f over the 2D domain described by mesh.

    f: function of (x, y) to integrate, mesh: dolfin.Mesh describing a
    triangulation of a 2D domain. Returns float.
    """
    # Fill in implementation below
    integral = 0.0
    for c in dolfin.cells(mesh):
        cell_integral = 0.0
        for v in dolfin.vertices(c):
            # Get coordinates
            x = v.point()[0]
            y = v.point()[1]
            # Get area/volume
            V = c.volume()

            cell_integral += ...
        integral += ...    

    return integral

def piecewise_linear_adapter(tarray, uarray):
    """Returns a piecewise linear interpolant function defined by
    arrays of t and u.

    tarray: float array of time values, uarray: float array of function values.
    Returns a function.
    """
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[i0]
        u1 = uarray[i1]

        # Lagrange polynomial weights
        w0 = (t1 - t) / (t1 - t0)
        w1 = (t - t0) / (t1 - t0)

        ut = w0*u0 + w1*u1
        return ut

    return vectorize(u_adapted)

def piecewise_constant_adapter(tarray, uarray):
    """Returns a piecewise constant interpolant function defined by
    arrays of t and u.

    tarray: float array of time values, uarray: float array of function values.
    Returns a function.
    """
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[i0]
        u1 = uarray[i1]

        ut = u1
        return ut

    return vectorize(u_adapted)

def interpolant_exponential():
    """Return interpolation points for question 2 subquestion 2.

    Returns array of interpolation points."""
    # Fill in implementation
    # You can use the following loop to see that the error estimate
    # is satisfied for all intervals

    def d2(t):
        # Second derivative of f
        return #?

    points = [ #? ]
    for i in range(0, size(points) - 1):
        # Size of subinterval
        h = points[i + 1] - points[i]
        t = points[i]
        d2i = d2(t)
        # Error estimate for interpolation
        interp_est = #?

        print "d2i: ", d2i
        print "interp_est: ", interp_est

    return points

def projection1D(mesh, f):
    """Computes the L2-projection of f into the piecewise linear
    function space defined by mesh.

    f_simple: function to be projected, mesh: dolfin.Mesh 1D interval
    Returns tuple of (tarray, Pf_array), where tarray is an array.
    """

    dx = dolfin.dx

    # Declare how our functions should be constructed. "CG" and 1 means
    # piecewise linear
    V = dolfin.FunctionSpace(mesh, "CG", 1)
    Pf = dolfin.TrialFunction(V)
    v = dolfin.TestFunction(V)

    # The function to be projected. Since it cannot be represented exactly
    # we choose a high-degree piecewise polynomial representation.
    f = dolfin.Expression("exp(-10*x[0])", degree=5)

    # Our equation to be solved: Pf*v*dx = f*v*dx, where "dx" means integration
    # over the domain.
    a = Pf*v*dx
    L = f*v*dx

    # Solve the equation
    problem = dolfin.VariationalProblem(a, L)
    Pf = problem.solve()

    # Extract arrays from the mesh and solution vector.
    # UnitInterval produces evenly spaced points, which we reproduce here.
    Pf_array = Pf.vector().array()
    tarray = linspace(0, 1, mesh.num_vertices())

    return (tarray, Pf_array)

def l2_norm(g, I):
    """Computes the L2-norm of the function g on the interval I.

    g: function to compute norm for, I: float tuple for start and end of interval
    Returns float of L2-norm value
    """
    def gsquare(t, u):
        return 0 # Fill in implementation

    return 0.0 # Fill in implementation

def f_1D(t):
    # Fill in
    return 0

def f_2D(x, y):
    # Fill in
    return 0

